#!/bin/bash
#

# build_geant4.sh <product_directory> <e4|e5> <debug|opt|prof> [tar]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e4|e5> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	build_type=Release
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
	build_type=Debug
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
	build_type=MinSizeRel
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

make_distribution()
{
    if [ "${maketar}" = "tar" ] 
    then
       if [ -d ${pkgdir}/lib64 ] || [ -d ${pkgdir}/lib ]
       then
	   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
	   exit 0
       fi
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=geant4
pkgver=v4_10_0_0_p02
ssibuildshims_version=v0_07_04
# deal with inconsistent naming 
# the G4 version is reported as 10.0.0
# but the tarball uses 10.00
G4version=10.0.0.p02
pkgtarver=4.10.00.p02
pkgtarfile=geant${pkgtarver}.tar.gz

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

make_distribution

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# pull the geant4 datasets
${thisdir}/scripts/getG4DataSets.sh ${product_dir}

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

set -x

cd ${sourcedir}
tar xzf ${tardir}/${pkgtarfile}  || die "ERROR: could not unwind ${pkgtarfile}"

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"
setup -B cmake v2_8_12_2 || die "ERROR: failed to setup cmake."

export GEANT4_SOURCE=${GEANT4_DIR}/source/geant${pkgtarver}
export BLDDIR=${blddir}
GEANT4_J=`${SSIBUILDSHIMS_DIR}/bin/ncores`

cd ${BLDDIR}
env CC=gcc CXX=g++ FC=gfortran cmake -DGEANT4_BUILD_CXXSTD="c++11" \
                                     -DCMAKE_INSTALL_PREFIX=${GEANT4_FQ_DIR} \
				     -DGEANT4_USE_SYSTEM_CLHEP=ON \
				     -DCLHEP_ROOT_DIR:STRING="${CLHEP_BASE_DIR}" \
				     -DBUILD_STATIC_LIBS=ON \
				     -DGEANT4_USE_OPENGL_X11=ON \
				     -DGEANT4_USE_GDML=ON \
				     -DXERCESC_ROOT_DIR:STRING="${XERCESCROOT}" \
				     -DCMAKE_BUILD_TYPE=${build_type} \
				     ${GEANT4_SOURCE}
(( $? == 0 )) || die "ERROR: cmake failed."
make ${GEANT4_J:+-j ${GEANT4_J}} || die "ERROR: make failed."
make install  || die "ERROR: make install failed."

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}

if [ ! -d ${G4LIB} ]
then
   echo "ERROR: failed to create ${G4LIB}"
   echo "       Something is very wrong"
   exit 1
fi
echo "G4LIB is ${G4LIB}"

# remove build directory from prof and opt distributions
if [ "${extraqual}" = "opt" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
elif [ "${extraqual}" = "prof" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
fi

# need to fix paths:
# bin/geant4-config
# lib64/Geant4-xxx/Geant4*.cmake
# share/Geant4-xxx/geant4make/geant4make*sh

fixfile=/tmp/g4fix$$$$.sh
rm -f $fixfile

echo "#!/bin/bash" > $fixfile
echo " set -x " >>  $fixfile
echo "mv ${GEANT4_FQ_DIR}/bin/geant4-config ${GEANT4_FQ_DIR}/bin/geant4-config.bak" >> $fixfile
echo "cat ${GEANT4_FQ_DIR}/bin/geant4-config.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%'  | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${GEANT4_FQ_DIR}/bin/geant4-config" >> $fixfile
echo "chmod +x  ${GEANT4_FQ_DIR}/bin/geant4-config" >> $fixfile

cmakeConfig=`ls ${G4LIB}/Geant4-${G4version}/Geant4*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> $fixfile
  echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%' | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%' > ${file}" >> $fixfile
done

g4make=`ls ${GEANT4_FQ_DIR}/share/Geant4-${G4version}/geant4make/geant4make*sh`
for file in ${g4make}
do
  echo "mv ${file} ${file}.bak" >> $fixfile
  echo "cat ${file}.bak | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%'  | sed 's%${XERCESCROOT}%\${XERCESCROOT}%' > ${file}" >> $fixfile
  echo "chmod +x  ${file}" >> $fixfile
done

chmod +x $fixfile
$fixfile

rm -f $fixfile

# this must be last
make_distribution

exit 0
